'''test_cr_lint.py: tests for the cr_lint module'''
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase
from unittest.mock import patch

from clickreviews.common import cleanup_unpack
from clickreviews.cr_lint import ClickReviewLint
from clickreviews.cr_lint import MINIMUM_CLICK_FRAMEWORK_VERSION
from clickreviews.tests import utils
import clickreviews.cr_tests as cr_tests

import os
import shutil
import stat
import tempfile


class TestClickReviewLint(cr_tests.TestClickReview):
    """Tests for the lint review tool."""

    def _create_hashes_yaml(self):
        # find cr_tests.py since that is what _get_statinfo() is mocked to
        # look at.
        f = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                         "../cr_tests.py")
        statinfo = os.stat(f)
        self.sha512 = cr_tests._get_sha512sum(self, f)
        hashes = {'archive-sha512': self.sha512,
                  'files': [{'name': 'bin',
                             'mode': 'drwxrwxr-x'},
                            {'name': 'bin/foo',
                             'size': statinfo.st_size,
                             'mode': 'f%s' %
                                     stat.filemode(statinfo.st_mode)[1:],
                             'sha512': self.sha512},
                            {'name': 'barlink',
                             'mode': 'lrwxrwxrwx'},
                            ]
                  }
        self._test_pkg_files = []
        for i in hashes['files']:
            self._test_pkg_files.append(i['name'])
        return hashes

    def patch_frameworks(self):
        def _mock_frameworks(self, overrides=None):
            self.FRAMEWORKS = {
                'ubuntu-core-15.04': 'available',
                'ubuntu-sdk-15.04': 'available',
                'ubuntu-sdk-13.10': 'deprecated',
                'ubuntu-sdk-14.10-qml-dev1': 'obsolete',
            }
            self.AVAILABLE_FRAMEWORKS = ['ubuntu-core-15.04',
                                         'ubuntu-sdk-15.04']
            self.OBSOLETE_FRAMEWORKS = ['ubuntu-sdk-14.10-qml-dev1']
            self.DEPRECATED_FRAMEWORKS = ['ubuntu-sdk-13.10']
        p = patch('clickreviews.frameworks.Frameworks.__init__',
                  _mock_frameworks)
        p.start()
        self.addCleanup(p.stop)

    def test_check_architecture(self):
        '''Test check_architecture()'''
        c = ClickReviewLint(self.test_name)
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_all(self):
        '''Test check_architecture_all() - no binaries'''
        self.set_test_control("Architecture", "all")
        self.set_test_manifest("architecture", "all")
        c = ClickReviewLint(self.test_name)
        c.pkg_bin_files = []
        c.check_architecture_all()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_all2(self):
        '''Test check_architecture_all() - binaries'''
        self.set_test_control("Architecture", "all")
        self.set_test_manifest("architecture", "all")
        c = ClickReviewLint(self.test_name)
        c.pkg_bin_files = ["path/to/some/compiled/binary"]
        c.check_architecture_all()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_architecture_armhf(self):
        '''Test check_architecture() - armhf'''
        self.set_test_control("Architecture", "armhf")
        c = ClickReviewLint(self.test_name)
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_i386(self):
        '''Test check_architecture() - i386'''
        self.set_test_control("Architecture", "i386")
        c = ClickReviewLint(self.test_name)
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_arm64(self):
        '''Test check_architecture() - arm64'''
        self.set_test_control("Architecture", "arm64")
        c = ClickReviewLint(self.test_name)
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_amd64(self):
        '''Test check_architecture() - amd64'''
        self.set_test_control("Architecture", "amd64")
        c = ClickReviewLint(self.test_name)
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_nonexistent(self):
        '''Test check_architecture() - nonexistent'''
        self.set_test_control("Architecture", "nonexistent")
        self.set_test_pkgfmt("click", "0.4")
        c = ClickReviewLint(self.test_name)
        c.pkg_arch = ["nonexistent"]
        c.check_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_control_architecture(self):
        '''Test check_control() (architecture)'''
        c = ClickReviewLint(self.test_name)
        c.check_control()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_architecture_snappy_1504(self):
        '''Test check_control() (architecture) - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        c.check_control()
        r = c.click_report
        expected_counts = {'info': 15, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_architecture_missing(self):
        '''Test check_control() (architecture missing)'''
        self.set_test_control("Architecture", None)
        try:
            ClickReviewLint(self.test_name)
        except KeyError:
            return
        raise Exception("Should have raised a KeyError")

    def test_check_control_matches_manifest_architecture(self):
        '''Test check_control() (architecture matches manifest)'''
        self.set_test_control("Architecture", "armhf")
        self.set_test_manifest("architecture", "armhf")
        c = ClickReviewLint(self.test_name)
        c.check_control()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_mismatches_manifest_architecture(self):
        '''Test check_control() (architecture mismatches manifest)'''
        self.set_test_control("Architecture", "armhf")
        self.set_test_manifest("architecture", "amd64")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_control()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_control_mismatches_manifest_architecture_snappy(self):
        '''Test check_control() (architecture mismatches manifest (snappy))'''
        self.set_test_control("Architecture", ["all"])
        self.set_test_manifest("architecture", "all")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_control()
        r = c.click_report
        expected_counts = {'info': 15, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_manifest_architecture_missing(self):
        '''Test check_control() (manifest architecture)'''
        self.set_test_control("Architecture", "armhf")
        self.set_test_manifest("architecture", None)
        c = ClickReviewLint(self.test_name)
        c.check_control()
        r = c.click_report

        expected = dict()
        expected['info'] = dict()
        expected['warn'] = dict()
        expected['error'] = dict()
        name = c._get_check_name('control_architecture_match')
        expected['info'][name] = {
            "text": "OK: architecture not specified in manifest"}
        self.check_results(r, expected=expected)

    def test_check_architecture_specified_needed(self):
        '''Test check_architecture_specified_needed() - no binaries'''
        self.set_test_control("Architecture", "armhf")
        self.set_test_manifest("architecture", "armhf")
        c = ClickReviewLint(self.test_name)
        c.pkg_arch = ['armhf']
        c.pkg_bin_files = []
        c.check_architecture_specified_needed()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_architecture_specified_needed2(self):
        '''Test check_architecture_specified_needed2() - binaries'''
        self.set_test_control("Architecture", "armhf")
        self.set_test_manifest("architecture", "armhf")
        c = ClickReviewLint(self.test_name)
        c.pkg_bin_files = ["path/to/some/compiled/binary"]
        c.check_architecture_specified_needed()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_missing_arch(self):
        '''Test check_manifest_architecture() (missing)'''
        self.set_test_manifest("architecture", None)
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_missing_arch_snappy_1504(self):
        '''Test check_manifest_architecture() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_manifest("architecture", None)
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_all(self):
        '''Test check_manifest_architecture() (all)'''
        self.set_test_manifest("architecture", "all")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_single_armhf(self):
        '''Test check_manifest_architecture() (single arch, armhf)'''
        self.set_test_manifest("architecture", "armhf")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_single_i386(self):
        '''Test check_manifest_architecture() (single arch, i386)'''
        self.set_test_manifest("architecture", "i386")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_single_amd64(self):
        '''Test check_manifest_architecture() (single arch, amd64)'''
        self.set_test_manifest("architecture", "amd64")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_single_nonexistent(self):
        '''Test check_manifest_architecture() (single nonexistent arch)'''
        self.set_test_manifest("architecture", "nonexistent")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_arch_single_multi(self):
        '''Test check_manifest_architecture() (single arch: invalid multi)'''
        self.set_test_manifest("architecture", "multi")
        c = ClickReviewLint(self.test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_valid_arch_multi(self):
        '''Test check_manifest_architecture() (valid multi)'''
        arch = "multi"
        self.set_test_manifest("architecture", ["armhf"])
        self.set_test_control("Architecture", arch)
        test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                        self.test_control['Version'],
                                        arch)
        c = ClickReviewLint(test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_valid_arch_multi2(self):
        '''Test check_manifest_architecture() (valid multi2)'''
        arch = "multi"
        self.set_test_manifest("architecture", ["armhf", "i386"])
        self.set_test_control("Architecture", arch)
        test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                        self.test_control['Version'],
                                        arch)
        c = ClickReviewLint(test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_invalid_arch_multi_nonexistent(self):
        '''Test check_manifest_architecture() (invalid multi)'''
        arch = "multi"
        self.set_test_manifest("architecture", ["armhf", "nonexistent"])
        self.set_test_control("Architecture", arch)
        test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                        self.test_control['Version'],
                                        arch)
        c = ClickReviewLint(test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_invalid_arch_multi_all(self):
        '''Test check_manifest_architecture() (invalid all)'''
        arch = "multi"
        self.set_test_manifest("architecture", ["armhf", "all"])
        self.set_test_control("Architecture", arch)
        test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                        self.test_control['Version'],
                                        arch)
        c = ClickReviewLint(test_name)
        c.is_snap1 = False
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_invalid_arch_multi_multi(self):
        '''Test check_manifest_architecture() (invalid multi)'''
        arch = "multi"
        self.set_test_manifest("architecture", ["multi", "armhf"])
        self.set_test_control("Architecture", arch)
        test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                        self.test_control['Version'],
                                        arch)
        c = ClickReviewLint(test_name)
        c.check_manifest_architecture()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_control(self):
        """A very basic test to make sure check_control can be tested."""
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_control()
        r = c.click_report
        expected_counts = {'info': 15, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_snap(self):
        """check_control with snap."""
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_control()
        r = c.click_report
        expected_counts = {'info': 15, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_control_snap_missing_maintainer(self):
        """check_control with snap with missing maintainer."""
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        self.set_test_control('Maintainer', None)
        c.check_control()
        r = c.click_report
        expected_counts = {'info': 15, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)
        # Lets check that the right info is triggering
        name = c._get_check_name('control_has_field:Maintainer')
        m = r['info'][name]['text']
        self.assertIn('OK (maintainer not required for snappy)', m)

    # Make the current MINIMUM_CLICK_FRAMEWORK_VERSION newer
    @patch('clickreviews.cr_lint.MINIMUM_CLICK_FRAMEWORK_VERSION',
           MINIMUM_CLICK_FRAMEWORK_VERSION + '.1')
    def test_check_control_click_framework_version(self):
        """Test that enforcing click framework versions works."""
        test_name = 'net.launchpad.click-webapps.test-app_3_all.click'
        c = ClickReviewLint(test_name)
        c.check_control()
        r = c.click_report
        # We should end up with an error as the click version is out of date
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        # Lets check that the right error is triggering
        name = c._get_check_name('control_click_version_up_to_date')
        m = r['error'][name]['text']
        self.assertIn('Click-Version is too old', m)

    def test_check_maintainer(self):
        '''Test check_maintainer()'''
        c = ClickReviewLint(self.test_name)
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_maintainer_empty(self):
        '''Test check_maintainer() - empty'''
        self.set_test_manifest("maintainer", "")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_maintainer_empty_snap(self):
        '''Test check_maintainer() - empty (snap)'''
        self.set_test_manifest("maintainer", "")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_maintainer_missing(self):
        '''Test check_maintainer() - missing (click)'''
        self.set_test_manifest("maintainer", None)
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_maintainer_missing_snap(self):
        '''Test check_maintainer() - missing (snap)'''
        self.set_test_manifest("maintainer", None)
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_maintainer_badformat(self):
        '''Test check_maintainer() - badly formatted'''
        self.set_test_manifest("maintainer", "$%^@*")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_maintainer_badformat_snap(self):
        '''Test check_maintainer() - badly formatted (snap)'''
        self.set_test_manifest("maintainer", "$%^@*")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_maintainer_bad_email_missing_name(self):
        '''Test check_maintainer() - bad email (missing name)'''
        self.set_test_manifest("name", "com.ubuntu.developer.user.app")
        self.set_test_manifest("maintainer",
                               "user@example.com")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_maintainer_bad_email_missing_name_snap(self):
        '''Test check_maintainer() - bad email (missing name, snap)'''
        self.set_test_manifest("name", "com.ubuntu.developer.user.app")
        self.set_test_manifest("maintainer",
                               "user@example.com")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_maintainer_domain_appstore(self):
        '''Test check_maintainer() - appstore domain
           (com.ubuntu.developer)'''
        self.set_test_manifest("name", "com.ubuntu.developer.user.app")
        self.set_test_manifest("maintainer",
                               "Foo User <user@example.com>")
        c = ClickReviewLint(self.test_name)
        c.check_maintainer()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_icon(self):
        '''Test check_icon()'''
        self.set_test_manifest("icon", "someicon")
        c = ClickReviewLint(self.test_name)
        c.check_icon()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_icon_snappy_1504(self):
        '''Test check_icon() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_manifest("icon", "someicon")
        c = ClickReviewLint(self.test_name)
        c.check_icon()
        r = c.click_report
        expected_counts = {'info': 3, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_icon_unspecified(self):
        '''Test check_icon()'''
        self.set_test_manifest("icon", None)
        c = ClickReviewLint(self.test_name)
        c.check_icon()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_icon_empty(self):
        '''Test check_icon() - empty'''
        self.set_test_manifest("icon", "")
        c = ClickReviewLint(self.test_name)
        c.check_icon()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_icon_absolute_path(self):
        '''Test check_icon() - absolute path'''
        self.set_test_manifest("icon", "/foo/bar/someicon")
        c = ClickReviewLint(self.test_name)
        c.check_icon()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_click_local_extensions_missing(self):
        '''Testeck_click_local_extensions() - missing'''
        for k in self.test_manifest.keys():
            if k.startswith("x-"):
                self.set_test_manifest(k, None)
        c = ClickReviewLint(self.test_name)
        c.check_click_local_extensions()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_click_local_extensions_empty(self):
        '''Testeck_click_local_extensions() - empty'''
        for k in self.test_manifest.keys():
            if k.startswith("x-"):
                self.set_test_manifest(k, None)
        self.set_test_manifest("x-test", "")
        c = ClickReviewLint(self.test_name)
        c.check_click_local_extensions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_click_local_extensions(self):
        '''Testeck_click_local_extensions()'''
        for k in self.test_manifest.keys():
            if k.startswith("x-"):
                self.set_test_manifest(k, None)
        self.set_test_manifest("x-source", {"vcs-bzr": "lp:notes-app",
                                            "vcs-bzr-revno": "209"})
        c = ClickReviewLint(self.test_name)
        c.check_click_local_extensions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_click_local_extensions_coreapp(self):
        '''Testeck_click_local_extensions() - coreapp'''
        for k in self.test_manifest.keys():
            if k.startswith("x-"):
                self.set_test_manifest(k, None)
        self.set_test_manifest("x-source", "foo")
        c = ClickReviewLint(self.test_name)
        c.is_core_app = True
        c.check_click_local_extensions()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_click_local_extensions_snappy_1504(self):
        '''Testeck_click_local_extensions() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        for k in self.test_manifest.keys():
            if k.startswith("x-"):
                self.set_test_manifest(k, None)
        self.set_test_manifest("x-source", {"vcs-bzr": "lp:notes-app",
                                            "vcs-bzr-revno": "209"})
        c = ClickReviewLint(self.test_name)
        c.check_click_local_extensions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework(self):
        '''Test check_framework()'''
        self.patch_frameworks()
        self.set_test_manifest("framework", "ubuntu-sdk-15.04")
        c = ClickReviewLint(self.test_name)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework_multiple_click(self):
        '''Test check_framework() - click'''
        self.patch_frameworks()
        self.set_test_manifest("framework",
                               "ubuntu-sdk-15.04,ubuntu-core-15.04")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_framework_multiple_snappy(self):
        '''Test check_framework() - snappy'''
        self.patch_frameworks()
        self.set_test_manifest("framework",
                               "ubuntu-sdk-15.04,ubuntu-core-15.04")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = True
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework_bad(self):
        '''Test check_framework() - bad'''
        self.patch_frameworks()
        self.set_test_manifest("framework", "nonexistent")
        c = ClickReviewLint(self.test_name)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_framework_deprecated(self):
        '''Test check_framework() - deprecated'''
        self.patch_frameworks()
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework_obsolete(self):
        '''Test check_framework() - obsolete'''
        self.patch_frameworks()
        self.set_test_manifest("framework", "ubuntu-sdk-14.10-qml-dev1")
        c = ClickReviewLint(self.test_name)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_framework_with_overrides(self):
        '''Test check_framework() - using overrides'''
        self.set_test_manifest("framework", "nonexistent")
        overrides = {'framework': {'nonexistent': {'state': 'available'}}}
        c = ClickReviewLint(self.test_name, overrides=overrides)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework_with_overrides_obsolete(self):
        '''Test check_framework() - using override obsoletes available'''
        fwk = 'ubuntu-sdk-15.04'
        self.set_test_manifest("framework", fwk)
        overrides = {'framework': {'%s' % fwk: {'state': 'obsolete'}}}
        c = ClickReviewLint(self.test_name, overrides=overrides)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_framework_with_overrides_deprecated(self):
        '''Test check_framework() - using override deprecates available'''
        fwk = 'ubuntu-sdk-15.04'
        self.set_test_manifest("framework", fwk)
        overrides = {'framework': {'%s' % fwk: {'state': 'deprecated'}}}
        c = ClickReviewLint(self.test_name, overrides=overrides)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_framework_with_malformed_overrides(self):
        '''Test check_framework() - using overrides'''
        self.set_test_manifest("framework", "nonexistent")
        overrides = {'nonexistent': {'state': 'available'}}
        c = ClickReviewLint(self.test_name, overrides=overrides)
        c.check_framework()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_hooks(self):
        '''Test check_hooks()'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_snappy_1504(self):
        '''Test check_hooks() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': 5, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_multiple_desktop_apps(self):
        '''Test check_hooks() - multiple desktop apps'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        tmp = c.manifest['hooks'][self.default_appname]
        c.manifest['hooks']["another-app"] = tmp
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': 9, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_multiple_apps(self):
        '''Test check_hooks() - multiple non-desktop apps'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        tmp = dict()
        for k in c.manifest['hooks'][self.default_appname].keys():
            tmp[k] = c.manifest['hooks'][self.default_appname][k]
        tmp.pop('desktop')
        tmp['scope'] = "some-scope-exec"
        c.manifest['hooks']["some-scope"] = tmp
        tmp = dict()
        for k in c.manifest['hooks'][self.default_appname].keys():
            tmp[k] = c.manifest['hooks'][self.default_appname][k]
        tmp.pop('desktop')
        tmp['push-helper'] = "push.json"
        c.manifest['hooks']["some-push-helper"] = tmp

        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': 13, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_security_extension(self):
        '''Test check_hooks() - security extension'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        tmp = dict()
        for k in c.manifest['hooks'][self.default_appname].keys():
            tmp[k] = c.manifest['hooks'][self.default_appname][k]
        tmp['apparmor'] = "%s.json" % self.default_appname
        c.manifest['hooks'][self.default_appname] = tmp

        c.check_hooks()
        r = c.click_report
        expected = dict()
        expected['info'] = dict()
        expected['warn'] = dict()
        expected['error'] = dict()
        name = c._get_check_name(
            'sdk_security_extension', app='test-app')
        expected['info'][name] = {
            "text": ("test-app.json does not end with .apparmor "
                     "(ok if not using sdk)")
        }
        self.check_results(r, expected=expected)

    def test_check_hooks_bad_appname(self):
        '''Test check_hooks() - bad appname'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        tmp = c.manifest['hooks'][self.default_appname]
        del c.manifest['hooks'][self.default_appname]
        c.manifest['hooks']["b@d@ppn@m#"] = tmp
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_hooks_missing_apparmor(self):
        '''Test check_hooks() - missing apparmor'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        del c.manifest['hooks'][self.default_appname]['apparmor']
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_hooks_missing_apparmor_with_apparmor_profile(self):
        '''Test check_hooks() - missing apparmor with apparmor-profile'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        del c.manifest['hooks'][self.default_appname]['apparmor']
        c.manifest['hooks'][self.default_appname]['apparmor-profile'] = 'foo'
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_missing_apparmor_with_puritine(self):
        '''Test check_hooks() - missing apparmor'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        del c.manifest['hooks'][self.default_appname]['apparmor']
        c.manifest['hooks'][self.default_appname]['puritine'] = 'foo'
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_has_desktop_and_scope(self):
        '''Test check_hooks() - desktop with scope'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["scope"] = "some-binary"
        c.check_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_hooks_type_oem(self):
        '''Test check_hooks() - type: oem'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.is_snap_oem = True
        c.check_hooks()
        r = c.click_report
        # oem type has no hooks so these should all be '0'
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_unknown_nonexistent(self):
        '''Test check_hooks_unknown() - nonexistent'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["nonexistant"] = "foo"
        c.check_hooks_unknown()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_unknown_nonexistent_snappy_1504(self):
        '''Test check_hooks_unknown() - nonexistent - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_hooks_unknown()
        r = c.click_report
        expected_counts = {'info': 3, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_unknown_good(self):
        '''Test check_hooks_unknown()'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_hooks_unknown()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_unknown_type_oem(self):
        '''Test check_hooks_unknown() - type: oem'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.is_snap_oem = True
        c.check_hooks_unknown()
        r = c.click_report
        # oem type has no hooks so these should all be '0'
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_redflagged_payui(self):
        '''Test check_hooks_redflagged() - pay-ui'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["pay-ui"] = "foo"
        c.check_hooks_redflagged()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('hooks_redflag', app='test-app')
        self.check_manual_review(r, name)

    def test_check_hooks_redflagged_payui_snappy_1504(self):
        '''Test check_hooks_redflagged() - pay-ui - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.check_hooks_redflagged()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_redflagged_apparmor_profile(self):
        '''Test check_hooks_redflagged() - apparmor-profile'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["apparmor-profile"] = "foo"
        # snap checks are handled elsewhere
        c.is_snap1 = False
        c.check_hooks_redflagged()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('hooks_redflag', app='test-app')
        self.check_manual_review(r, name)

    def test_check_hooks_redflagged_puritine(self):
        '''Test check_hooks_redflagged() - puritine'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["puritine"] = "foo"
        c.check_hooks_redflagged()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('hooks_redflag', app='test-app')
        self.check_manual_review(r, name)

    def test_pkgname_toplevel(self):
        '''Test check_pkgname - toplevel'''
        self.set_test_manifest("name", "foo")
        c = ClickReviewLint(self.test_name)
        c.check_pkgname()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_pkgname_flat(self):
        '''Test check_pkgname - flat'''
        self.set_test_manifest("name", "foo.bar")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_pkgname()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_pkgname_reverse_domain(self):
        '''Test check_pkgname - reverse domain'''
        self.set_test_manifest("name", "com.ubuntu.develeper.baz.foo")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_pkgname()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_name_toplevel_1504(self):
        '''Test check_snappy_name - toplevel - 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "foo")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_name_flat(self):
        '''Test check_snappy_name - obsoleted flat'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "foo.bar")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_name_reverse_domain(self):
        '''Test check_snappy_name - obsoleted reverse domain'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "com.ubuntu.develeper.baz.foo")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_name_bad(self):
        '''Test check_snappy_name - bad'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "foo?bar")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_name_bad2(self):
        '''Test check_snappy_name - empty'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_name_bad3(self):
        '''Test check_snappy_name - list'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", [])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_name_bad4(self):
        '''Test check_snappy_name - dict'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", {})
        c = ClickReviewLint(self.test_name)
        c.check_snappy_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_version_1504(self):
        '''Test check_snappy_version - 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", 1)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version1(self):
        '''Test check_snappy_version - integer'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", 1)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version2(self):
        '''Test check_snappy_version - float'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", 1.0)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version3(self):
        '''Test check_snappy_version - MAJOR.MINOR.MICRO'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", "1.0.1")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version4(self):
        '''Test check_snappy_version - str'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", "1.0a")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version5(self):
        '''Test check_snappy_version - alpha'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", "a.b")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_version_bad(self):
        '''Test check_snappy_version - bad'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", "foo?bar")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_version_bad2(self):
        '''Test check_snappy_version - empty'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", "")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_version_bad3(self):
        '''Test check_snappy_version - list'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", [])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_version_bad4(self):
        '''Test check_snappy_version - dict'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("version", {})
        c = ClickReviewLint(self.test_name)
        c.check_snappy_version()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_type(self):
        '''Test check_snappy_type - unspecified'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", None)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_app_1504(self):
        '''Test check_snappy_type - app - 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "app")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_framework(self):
        '''Test check_snappy_type - framework'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_oem(self):
        '''Test check_snappy_type - oem'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "oem")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_redflagged_1504(self):
        '''Test check_snappy_type_redflagged - unspecified - 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", None)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type_redflagged()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_redflagged_app(self):
        '''Test check_snappy_type_redflagged - app'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "app")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type_redflagged()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_type_redflagged_framework(self):
        '''Test check_snappy_type_redflagged - framework'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type_redflagged()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_type_redflagged_oem(self):
        '''Test check_snappy_type_redflagged - oem'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("type", "oem")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_type_redflagged()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_icon_1504(self):
        '''Test check_snappy_icon() - 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("icon", "someicon")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_icon()
        r = c.click_report
        expected_counts = {'info': 3, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_icon_unspecified(self):
        '''Test check_snappy_icon() - unspecified'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("icon", None)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_icon()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_icon_empty(self):
        '''Test check_snappy_icon() - empty'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("icon", "")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_icon()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_icon_absolute_path(self):
        '''Test check_snappy_icon() - absolute path'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("icon", "/foo/bar/someicon")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_icon()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_missing_arch(self):
        '''Test check_snappy_architecture() (missing)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", None)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_all_deprecated(self):
        '''Test check_snappy_architecture() (deprecated, all)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architecture", "all")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_amd64_deprecated(self):
        '''Test check_snappy_architecture() (deprecated, all)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architecture", "amd64")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_all(self):
        '''Test check_snappy_architecture() (all)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["all"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_armhf(self):
        '''Test check_snappy_architecture() (single arch, armhf)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["armhf"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_arm64(self):
        '''Test check_snappy_architecture() (single arch, arm64)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["arm64"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_i386(self):
        '''Test check_snappy_architecture() (single arch, i386)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["i386"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_amd64(self):
        '''Test check_snappy_architecture() (single arch, amd64)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["amd64"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_nonexistent(self):
        '''Test check_snappy_architecture() (single nonexistent arch)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["nonexistent"])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_arch_single_multi(self):
        '''Test check_snappy_architecture() (single arch: invalid multi)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", "multi")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_valid_arch_multi(self):
        '''Test check_snappy_architecture() (valid multi)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("architectures", ["armhf"])
        test_name = "%s_%s_%s.snap" % (self.test_control['Package'],
                                       self.test_control['Version'],
                                       "armhf")
        c = ClickReviewLint(test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_valid_arch_multi2(self):
        '''Test check_snappy_architecture() (valid multi2)'''
        self.set_test_pkgfmt("snap", "15.04")
        arch = "multi"
        self.set_test_pkg_yaml("architectures", ["armhf", "i386"])
        test_name = "%s_%s_%s.snap" % (self.test_control['Package'],
                                       self.test_control['Version'],
                                       arch)
        c = ClickReviewLint(test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_invalid_arch_multi_nonexistent(self):
        '''Test check_snappy_architecture() (invalid multi)'''
        self.set_test_pkgfmt("snap", "15.04")
        arch = "multi"
        self.set_test_pkg_yaml("architectures", ["armhf", "nonexistent"])
        test_name = "%s_%s_%s.snap" % (self.test_control['Package'],
                                       self.test_control['Version'],
                                       arch)
        c = ClickReviewLint(test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_invalid_arch_multi_all(self):
        '''Test check_snappy_architecture() (invalid all)'''
        self.set_test_pkgfmt("snap", "15.04")
        arch = "multi"
        self.set_test_pkg_yaml("architectures", ["armhf", "all"])
        test_name = "%s_%s_%s.snap" % (self.test_control['Package'],
                                       self.test_control['Version'],
                                       arch)
        c = ClickReviewLint(test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_invalid_arch_multi_multi(self):
        '''Test check_snappy_architecture() (invalid multi)'''
        self.set_test_pkgfmt("snap", "15.04")
        arch = "multi"
        self.set_test_pkg_yaml("architectures", ["multi", "armhf"])
        test_name = "%s_%s_%s.snap" % (self.test_control['Package'],
                                       self.test_control['Version'],
                                       arch)
        c = ClickReviewLint(test_name)
        c.check_snappy_architecture()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown_entries(self):
        '''Test check_snappy_unknown_entries - none'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "foo")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_unknown_entries()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown_entries2(self):
        '''Test check_snappy_unknown_entries - one'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("nonexistent", "bar")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_unknown_entries()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown_obsoleted(self):
        '''Test check_snappy_unknown_entries - obsoleted'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("maintainer", "bar")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_unknown_entries()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        # Lets check that the right warning is triggering
        name = c._get_check_name('snappy_unknown')
        m = r['error'][name]['text']
        self.assertIn("unknown entries in package.yaml: 'maintainer' "
                      "(maintainer obsoleted)", m)

    def test_check_snappy_readme_md(self):
        '''Test check_snappy_readme_md()'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_readme_md("%s - some description" %
                                self.test_name.split('_')[0])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_readme_md()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_readme_md_bad(self):
        '''Test check_snappy_readme_md() - short'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", "prettylong.name")
        self.set_test_readme_md("abc")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_readme_md()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_readme_md_bad2(self):
        '''Test check_snappy_readme_md() - missing'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_readme_md(None)
        c = ClickReviewLint(self.test_name)
        c.check_snappy_readme_md()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_services_and_binaries1(self):
        '''Test check_snappy_services_and_binaries() - different'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_pkg_yaml("services", [{"name": "foo",
                                             "start": "bin/foo"}])
        self.set_test_pkg_yaml("binaries", [{"name": "bar"}])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_services_and_binaries()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_services_and_binaries2(self):
        '''Test check_snappy_services_and_binaries() - different (exec)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_pkg_yaml("services", [{"name": "foo",
                                             "start": "bin/foo"}])
        self.set_test_pkg_yaml("binaries", [{"name": "bar",
                                             "exec": "bin/foo"}])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_services_and_binaries()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_services_and_binaries3(self):
        '''Test check_snappy_services_and_binaries() - same'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_pkg_yaml("services", [{"name": "foo",
                                             "start": "bin/foo"}])
        self.set_test_pkg_yaml("binaries", [{"name": "foo"}])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_services_and_binaries()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

        name = c._get_check_name('snappy_in_services', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)
        name = c._get_check_name('snappy_in_binaries', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)

    def test_check_snappy_services_and_binaries4(self):
        '''Test check_snappy_services_and_binaries() - same (subdir)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_pkg_yaml("services", [{"name": "foo",
                                             "start": "bin/foo"}])
        self.set_test_pkg_yaml("binaries", [{"name": "bin/foo"}])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_services_and_binaries()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

        name = c._get_check_name('snappy_in_services', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)
        name = c._get_check_name('snappy_in_binaries', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)

    def test_check_snappy_services_and_binaries5(self):
        '''Test check_snappy_services_and_binaries() - same (exec, subdir)'''
        self.set_test_pkgfmt("snap", "15.04")
        self.set_test_pkg_yaml("name", self.test_name.split('_')[0])
        self.set_test_pkg_yaml("services", [{"name": "foo",
                                             "start": "bin/foo"}])
        self.set_test_pkg_yaml("binaries", [{"name": "foo",
                                             "exec": "bin/foo"}])
        c = ClickReviewLint(self.test_name)
        c.check_snappy_services_and_binaries()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

        name = c._get_check_name('snappy_in_services', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)
        name = c._get_check_name('snappy_in_binaries', extra='foo')
        m = r['error'][name]['text']
        self.assertIn("'foo' in both 'services' and 'binaries'", m)

    def test_check_snappy_hashes_click(self):
        '''Test check_snappy_hashes() - click'''
        self.set_test_pkgfmt("click", "0.4")
        c = ClickReviewLint(self.test_name)
        c.is_snap1 = False
        c.check_snappy_hashes()
        r = c.click_report
        # clicks don't have hashes.yaml, so should have no output
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_sha512_missing(self):
        '''Test check_snappy_hashes() - archive-sha512 missing'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_sha512_invalid(self):
        '''Test check_snappy_hashes() - archive-sha512 invalid'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        yaml['archive-sha512'] = 'deadbeef'
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('hashes_archive-sha512_valid')
        m = r['error'][name]['text']
        self.assertIn("hash mismatch: 'deadbeef' != '%s'" % self.sha512, m)

    def test_check_snappy_hashes_archive_files_missing(self):
        '''Test check_snappy_hashes() - files missing'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        del yaml['files']
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('hashes_files_present')
        m = r['error'][name]['text']
        self.assertIn("'files' not found in hashes.yaml", m)

    # def test_check_snappy_hashes_archive_files_ok(self):
    #     '''Test check_snappy_hashes() - ok'''
    #     self.set_test_pkgfmt("snap", "15.04")
    #     c = ClickReviewLint(self.test_name)
    #     yaml = self._create_hashes_yaml()
    #     c.pkg_files = self._test_pkg_files
    #     self.set_test_hashes_yaml(yaml)
    #     c.check_snappy_hashes()
    #     r = c.click_report
    #     expected_counts = {'info': 4, 'warn': 0, 'error': 0}
    #     self.check_results(r, expected_counts)

    # def test_check_snappy_hashes_1504(self):
    #     '''Test check_snappy_hashes() - 15.04'''
    #     self.set_test_pkgfmt("snap", "15.04")
    #     c = ClickReviewLint(self.test_name)
    #     yaml = self._create_hashes_yaml()
    #     c.pkg_files = self._test_pkg_files
    #     self.set_test_hashes_yaml(yaml)
    #     c.check_snappy_hashes()
    #     r = c.click_report
    #     expected_counts = {'info': 4, 'warn': 0, 'error': 0}
    #     self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_files_missing_name(self):
        '''Test check_snappy_hashes() - missing name'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        del yaml['files'][0]['name']
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_files_missing_mode(self):
        '''Test check_snappy_hashes() - missing mode'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        del yaml['files'][0]['mode']
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_files_malformed_mode(self):
        '''Test check_snappy_hashes() - malformed mode'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        yaml['files'][0]['mode'] += 'extra'
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_files_bad_mode_entry(self):
        '''Test check_snappy_hashes() - bad mode entry'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        orig_mode = None
        for e in yaml['files']:
            s = list(yaml['files'][count]['mode'])
            if e['name'] == 'bin/foo':
                # keep track of the other parts of the on disk mode
                orig_mode = s
                orig_mode[3] = 'S'
                s[3] = 'S'
            yaml['files'][count]['mode'] = "".join(s)
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('file_mode')
        m = r['error'][name]['text']
        self.assertIn("found errors in hashes.yaml: unusual mode '%s' "
                      "for entry 'bin/foo'" % "".join(orig_mode), m)

    def test_check_snappy_hashes_archive_files_mode_world_write(self):
        '''Test check_snappy_hashes() - mode world write'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        for e in yaml['files']:
            s = list(e['mode'])
            if e['name'] == 'bin/foo' or e['name'] == 'bin':
                s[-2] = 'w'
                s[-5] = 'w'
            yaml['files'][count]['mode'] = "".join(s)
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('file_mode')
        m = r['error'][name]['text']
        self.assertIn("found errors in hashes.yaml: 'bin' is world-writable, "
                      "mode 'frw-rw-rw-' for 'bin/foo' is world-writable", m)

    def test_check_snappy_hashes_archive_files_mode_mismatch(self):
        '''Test check_snappy_hashes() - mode mismatch'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        orig_mode = None
        for e in yaml['files']:
            if e['mode'].startswith('f'):
                # keep track of the other parts of the on disk mode
                orig_mode = e['mode'][1:]
                yaml['files'][count]['mode'] = "f---------"
                break
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('file_mode')
        m = r['error'][name]['text']
        self.assertIn("found errors in hashes.yaml: mode '---------' != '%s' "
                      "for 'bin/foo'" % orig_mode, m)

    # def test_check_snappy_hashes_archive_files_mode_bad_symlink(self):
    #     '''Test check_snappy_hashes() - mode bad symlink'''
    #     self.set_test_pkgfmt("snap", "15.04")
    #     c = ClickReviewLint(self.test_name)
    #     yaml = self._create_hashes_yaml()
    #     yaml['files'].append({'name': 'badlink', 'mode': 'lrwxrwxr-x'})
    #     self.set_test_hashes_yaml(yaml)
    #     c.check_snappy_hashes()
    #     r = c.click_report
    #     expected_counts = {'info': None, 'warn': 0, 'error': 1}
    #     self.check_results(r, expected_counts)
    #     name = c._get_check_name('file_mode')
    #     m = r['error'][name]['text']
    #     self.assertIn("found errors in hashes.yaml: unusual mode 'lrwxrwxr-x' "
    #                   "for entry 'badlink'", m)

    # def test_check_snappy_hashes_archive_files_mode_devices(self):
    #     '''Test check_snappy_hashes() - mode devices'''
    #     self.set_test_pkgfmt("snap", "15.04")
    #     c = ClickReviewLint(self.test_name)
    #     yaml = self._create_hashes_yaml()
    #     yaml['files'].append({'name': 'badblock', 'mode': 'brw-rw-r--'})
    #     yaml['files'].append({'name': 'badchar', 'mode': 'crw-rw-r--'})
    #     self.set_test_hashes_yaml(yaml)
    #     c.check_snappy_hashes()
    #     r = c.click_report
    #     expected_counts = {'info': None, 'warn': 0, 'error': 1}
    #     self.check_results(r, expected_counts)
    #     name = c._get_check_name('file_mode')
    #     m = r['error'][name]['text']
    #     self.assertIn("found errors in hashes.yaml: "
    #                   "illegal file mode 'b': 'brw-rw-r--' for 'badblock', "
    #                   "illegal file mode 'c': 'crw-rw-r--' for 'badchar'", m)

    def test_check_snappy_hashes_archive_files_missing_size(self):
        '''Test check_snappy_hashes() - missing size'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        for e in yaml['files']:
            if e['mode'].startswith('f'):
                del yaml['files'][count]['size']
                break
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_hashes_archive_files_invalid_size(self):
        '''Test check_snappy_hashes() - invalid size'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        for e in yaml['files']:
            if e['mode'].startswith('f'):
                orig_size = e['size']
                new_size = orig_size + 1
                yaml['files'][count]['size'] = new_size
                break
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)
        name = c._get_check_name('file_mode')
        m = r['error'][name]['text']
        self.assertIn("found errors in hashes.yaml: size " +
                      "%d != %d for 'bin/foo'" % (new_size, orig_size), m)

    def test_check_snappy_hashes_archive_files_missing_sha512(self):
        '''Test check_snappy_hashes() - missing sha512'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        yaml = self._create_hashes_yaml()
        count = 0
        for e in yaml['files']:
            if e['mode'].startswith('f'):
                del yaml['files'][count]['sha512']
                break
            count += 1
        self.set_test_hashes_yaml(yaml)
        c.check_snappy_hashes()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    # def test_check_snappy_hashes_extra(self):
    #     '''Test check_snappy_hashes() - extra'''
    #     self.set_test_pkgfmt("snap", "15.04")
    #     c = ClickReviewLint(self.test_name)
    #     yaml = self._create_hashes_yaml()
    #     self.set_test_hashes_yaml(yaml)
    #     c.pkg_files = self._test_pkg_files
    #     c.pkg_files.append("extrafile")
    #     c.check_snappy_hashes()
    #     r = c.click_report
    #     expected_counts = {'info': None, 'warn': 0, 'error': 1}
    #     self.check_results(r, expected_counts)
    #     name = c._get_check_name('hashes_extra_files')
    #     m = r['error'][name]['text']
    #     self.assertIn("found extra files not listed in hashes.yaml: extrafile",
    #                   m)

    def test_snappy_config(self):
        '''Test check_snappy_config()'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        c.unpack_dir = "/nonexistent"
        c.pkg_files.append(os.path.join(c.unpack_dir, 'meta/hooks/config'))
        c.check_snappy_config()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_config_nonexecutable(self):
        '''Test check_snappy_config() - not executable'''
        self.set_test_pkgfmt("snap", "15.04")
        c = ClickReviewLint(self.test_name)
        c.unpack_dir = "/nonexistent.nonexec"
        c.pkg_files.append(os.path.join(c.unpack_dir,
                                        'meta/hooks/config'))
        c.check_snappy_config()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_external_symlinks_puritine(self):
        '''Test check_external_symlinks - puritine'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["puritine"] = "foo"
        c.check_external_symlinks()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)
        name = c._get_check_name('external_symlinks')
        m = r['info'][name]['text']
        self.assertIn("SKIPPED: puritine", m)

    def test_check_md5sums_puritine(self):
        '''Test check_md5sums - puritine'''
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        c = ClickReviewLint(self.test_name)
        c.manifest['hooks'][self.default_appname]["puritine"] = "foo"
        c.check_md5sums()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)
        name = c._get_check_name('md5sums')
        m = r['info'][name]['text']
        self.assertIn("SKIPPED: puritine", m)


class ClickReviewLintTestCase(TestCase):
    """Tests without mocks where they are not needed."""
    def setUp(self):
        # XXX cleanup_unpack() is required because global variables
        # UNPACK_DIR, RAW_UNPACK_DIR are initialised to None at module
        # load time, but updated when a real (non-Mock) test runs, such as
        # here. While, at the same time, two of the existing tests using
        # mocks depend on both global vars being None. Ideally, those
        # global vars should be refactored away.
        self.addCleanup(cleanup_unpack)
        super().setUp()

    def mkdtemp(self):
        """Create a temp dir which is cleaned up after test."""
        tmp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmp_dir)
        return tmp_dir

    def test_check_dot_click_root(self):
        package = utils.make_click(extra_files=['.click/'],
                                   output_dir=self.mkdtemp())
        c = ClickReviewLint(package)

        c.check_dot_click()

        errors = list(c.click_report['error'].keys())
        self.assertEqual(errors, ['lint:dot_click'])



'''cr_push_helper.py: click push-helper checks'''
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import json
import os


class ClickReviewPushHelper(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        my_hook = 'push-helper'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ['apparmor']
        peer_hooks[my_hook]['required'] = ['apparmor']

        ClickReview.__init__(self, fn, "push_helper", peer_hooks=peer_hooks,
                             overrides=overrides)

        if not self.is_click and not self.is_snap1:
            return

        self.required_keys = ['exec']
        self.optional_keys = ['app_id']

        self.push_helper_files = dict()  # click-show-files and tests
        self.push_helper = dict()

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            if 'push-helper' not in self.manifest['hooks'][app]:
                # msg("Skipped missing push-helper hook for '%s'" % app)
                continue
            if not isinstance(self.manifest['hooks'][app]['push-helper'], str):
                error("manifest malformed: hooks/%s/push-helper is not str" %
                      app)
            (full_fn, jd) = self._extract_push_helper(app)
            self.push_helper_files[app] = full_fn
            self.push_helper[app] = jd

    def _extract_push_helper(self, app):
        '''Get push-helper hook content'''
        c = self.manifest['hooks'][app]['push-helper']
        fn = os.path.join(self.unpack_dir, c)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        fh = open_file_read(fn)
        contents = ""
        for line in fh.readlines():
            contents += line
        fh.close()

        try:
            jd = json.loads(contents)
        except Exception as e:
            error("push-helper json unparseable: %s (%s):\n%s" % (bn,
                  str(e), contents))

        if not isinstance(jd, dict):
            error("push-helper json is malformed: %s:\n%s" % (bn, contents))

        return (fn, jd)

    def check_valid(self):
        '''Check validity of push-helper entries'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.push_helper):
            for k in self.push_helper[app].keys():
                t = "info"
                n = self._get_check_name("valid", app=app, extra=k)
                s = "OK"

                if not isinstance(self.push_helper[app][k], str):
                    t = "error"
                    s = "'%s' is not a string" % k
                elif self.push_helper[app][k] == "":
                    t = "error"
                    s = "'%s' is empty" % k
                self._add_result(t, n, s)

            for k in self.required_keys:
                t = "info"
                n = self._get_check_name("valid_required", app=app, extra=k)
                s = "OK"
                if k not in self.push_helper[app]:
                    t = "error"
                    s = "'%s' is missing" % k
                self._add_result(t, n, s)

    def check_unknown_keys(self):
        '''Check unknown'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.push_helper):
            unknown = []
            t = "info"
            n = self._get_check_name("unknown", app=app)
            s = "OK"
            for key in self.push_helper[app].keys():
                if key not in self.required_keys and \
                   key not in self.optional_keys:
                    unknown.append(key)
            if len(unknown) == 1:
                t = "warn"
                s = "Unknown field '%s'" % unknown[0]
            elif len(unknown) > 1:
                t = "warn"
                s = "Unknown fields '%s'" % ", ".join(unknown)
            self._add_result(t, n, s)

    def check_hooks(self):
        '''Verify combinations of click hooks with the push-helper hook'''
        if not self.is_click and not self.is_snap1:
            return

        if self.manifest is None:
            return

        for app in sorted(self.manifest['hooks']):
            if app not in self.push_helper:
                continue

            t = "info"
            n = self._get_check_name("other_hooks", app=app)
            s = "OK"

            bad = []
            for hook in self.manifest['hooks'][app]:
                # Only the apparmor hook can be used with the push-helper
                if hook not in ['push-helper', 'apparmor']:
                    bad.append(hook)
            if len(bad) > 0:
                t = 'error'
                s = "Found unusual hooks with push-helper: %s" % ", ".join(bad)
            self._add_result(t, n, s)



'''cr_url dispatcher.py: click url_dispatcher'''
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import json
import os

# https://wiki.ubuntu.com/URLDispatcher


class ClickReviewUrlDispatcher(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        my_hook = 'urls'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ClickReview.app_allowed_peer_hooks
        peer_hooks[my_hook]['required'] = []

        ClickReview.__init__(self, fn, "url_dispatcher", peer_hooks=peer_hooks,
                             overrides=overrides)

        if not self.is_click and not self.is_snap1:
            return

        self.required_keys = ['protocol']
        self.optional_keys = ['domain-suffix']

        self.url_dispatcher_files = dict()  # click-show-files and tests
        self.url_dispatcher = dict()

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            if 'urls' not in self.manifest['hooks'][app]:
                # msg("Skipped missing urls hook for '%s'" % app)
                continue
            if not isinstance(self.manifest['hooks'][app]['urls'], str):
                error("manifest malformed: hooks/%s/urls is not str" % app)
            (full_fn, jd) = self._extract_url_dispatcher(app)
            self.url_dispatcher_files[app] = full_fn
            self.url_dispatcher[app] = jd

    def _extract_url_dispatcher(self, app):
        '''Get url dispatcher json'''
        u = self.manifest['hooks'][app]['urls']
        if not u:
            error("'urls' definition is empty for '%s'" % app)
        fn = os.path.join(self.unpack_dir, u)
        if not os.path.isfile(fn):
            error("'%s' is not a file" % fn)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        fh = open_file_read(fn)
        contents = ""
        for line in fh.readlines():
            contents += line
        fh.close()

        try:
            jd = json.loads(contents)
        except Exception as e:
            error("url-dispatcher json unparseable: %s (%s):\n%s" % (bn,
                  str(e), contents))

        if not isinstance(jd, list):
            error("url-dispatcher json is malformed: %s:\n%s" % (bn, contents))

        return (fn, jd)

    def check_required(self):
        '''Check url-dispatcher required fields'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.url_dispatcher):
            for r in self.required_keys:
                found = False
                t = 'info'
                n = self._get_check_name('required_entry', app=app, extra=r)
                s = "OK"
                for entry in self.url_dispatcher[app]:
                    if not isinstance(entry, dict):
                        t = 'error'
                        s = "'%s' is not a dict" % str(entry)
                        self._add_result(t, n, s)
                        continue
                    if r in entry:
                        if not isinstance(entry[r], str):
                            t = 'error'
                            s = "'%s' is not a string" % r
                        elif entry[r] == "":
                            t = 'error'
                            s = "'%s' is empty" % r
                        else:
                            found = True
                if not found and t != 'error':
                    t = 'error'
                    s = "Missing required field '%s'" % r
                self._add_result(t, n, s)

    def check_optional(self):
        '''Check url-dispatcher optional fields'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.url_dispatcher):
            for o in self.optional_keys:
                found = False
                t = 'info'
                n = self._get_check_name('optional_entry', app=app, extra=o)
                s = "OK"
                for entry in self.url_dispatcher[app]:
                    if not isinstance(entry, dict):
                        t = 'error'
                        s = "'%s' is not a dict" % str(entry)
                        self._add_result(t, n, s)
                        continue
                    if o in entry:
                        if not isinstance(entry[o], str):
                            t = 'error'
                            s = "'%s' is not a string" % o
                        elif entry[o] == "":
                            t = 'error'
                            s = "'%s' is empty" % o
                        else:
                            found = True
                if not found and t != 'error':
                    s = "OK (skip missing)"
                self._add_result(t, n, s)

    def check_unknown(self):
        '''Check url-dispatcher unknown fields'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.url_dispatcher):
            unknown = []
            for entry in self.url_dispatcher[app]:
                t = 'info'
                n = self._get_check_name('unknown_entry', app=app)
                s = "OK"
                if not isinstance(entry, dict):
                    t = 'error'
                    s = "'%s' is not a dict" % str(entry)
                    self._add_result(t, n, s)
                    continue

                for f in entry.keys():
                    if f not in self.required_keys and \
                       f not in self.optional_keys:
                        unknown.append(f)

            if len(unknown) == 1:
                t = 'warn'
                s = "Unknown field '%s'" % unknown[0]
            elif len(unknown) > 1:
                t = 'warn'
                s = "Unknown fields '%s'" % ", ".join(unknown)
            self._add_result(t, n, s)



#! /usr/bin/env python3

from setuptools import setup, find_packages
import codecs
import glob
import os
import re

# look/set what version we have
changelog = 'debian/changelog'
if os.path.exists(changelog):
    head = codecs.open(changelog, encoding='utf-8').readline()
    match = re.compile('.*\((.*)\).*').match(head)
    if match:
        version = match.group(1)

scripts = glob.glob('bin/click-*')
scripts.remove('bin/click-check-skeleton')

requirements = [
    'python-magic',
    'PyYaml',
    'lxml',
    'pyxdg',
    'python-apt',
    'python-debian',
]

setup(
    name='click-reviewers-tools',
    version=version,
    scripts=scripts,
    packages=find_packages(),
    test_suite='clickreviews.tests',
    install_requires=requirements,
)



'''cr_desktop.py: click desktop checks'''
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import glob
import json
import os
import re
from urllib.parse import urlsplit
from xdg.DesktopEntry import DesktopEntry
from xdg.Exceptions import ParsingError as xdgParsingError


class ClickReviewDesktop(ClickReview):
    '''This class represents click lint reviews'''
    supported_versions = ("1.0", "1.1", "1.2", "1.3", "1.4", "1.5")

    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        my_hook = 'desktop'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ClickReview.app_allowed_peer_hooks
        peer_hooks[my_hook]['required'] = ["apparmor"]

        ClickReview.__init__(self, fn, "desktop", peer_hooks=peer_hooks,
                             overrides=overrides)
        if not self.is_click and not self.is_snap1:
            return

        self.desktop_files = dict()  # click-show-files and a couple tests
        self.desktop_entries = dict()
        self.desktop_hook_entries = 0

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            if 'desktop' not in self.manifest['hooks'][app]:
                # msg("Skipped missing desktop hook for '%s'" % app)
                continue
            if not isinstance(self.manifest['hooks'][app]['desktop'], str):
                error("manifest malformed: hooks/%s/desktop is not str" % app)
            self.desktop_hook_entries += 1
            (de, full_fn) = self._extract_desktop_entry(app)
            self.desktop_entries[app] = de
            self.desktop_files[app] = full_fn

        self.required_keys = ['Name',
                              'Type',
                              'Icon',
                              'Exec',
                              'X-Ubuntu-Touch',
                              ]
        self.expected_execs = ['qmlscene',
                               'webbrowser-app',
                               'webapp-container',
                               'ubuntu-html5-app-launcher',
                               ]
        self.deprecated_execs = ['cordova-ubuntu-2.8',
                                 ]
        # TODO: the desktop hook will actually handle this correctly
        self.blacklisted_keys = ['Path']

    def _extract_desktop_entry(self, app):
        '''Get DesktopEntry for desktop file and verify it'''
        d = self.manifest['hooks'][app]['desktop']
        fn = os.path.join(self.unpack_dir, d)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        fh = open_file_read(fn)
        contents = ""
        for line in fh.readlines():
            contents += line
        fh.close()

        try:
            de = DesktopEntry(fn)
        except xdgParsingError as e:
            error("desktop file unparseable: %s (%s):\n%s" % (bn, str(e),
                                                              contents))
        try:
            de.parse(fn)
        except Exception as e:
            error("desktop file unparseable: %s (%s):\n%s" % (bn, str(e),
                                                              contents))
        return de, fn

    def _get_desktop_entry(self, app):
        '''Get DesktopEntry from parsed values'''
        return self.desktop_entries[app]

    def _get_desktop_files(self):
        '''Get desktop_files (abstracted out for mock)'''
        return self.desktop_files

    def _get_desktop_filename(self, app):
        '''Get desktop file filenames'''
        return self.desktop_files[app]

    def check_desktop_file(self):
        '''Check desktop file'''
        if not self.is_click and not self.is_snap1:
            return

        t = 'info'
        n = self._get_check_name('files_usable')
        s = 'OK'
        if len(self._get_desktop_files().keys()) != self.desktop_hook_entries:
            t = 'error'
            s = 'Could not use all specified .desktop files'
        elif self.desktop_hook_entries == 0:
            s = 'Skipped: could not find any desktop files'
        self._add_result(t, n, s)

    def check_desktop_file_valid(self):
        '''Check desktop file validates'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('validates', app=app)
            s = 'OK'
            link = None
            try:
                de.validate()
            except Exception as e:
                t = 'error'
                s = 'did not validate: (%s)' % str(e)
                link = ('http://askubuntu.com/questions/417377/'
                        'what-does-desktop-validates-mean/417378')
            self._add_result(t, n, s, link)

    def check_desktop_required_keys(self):
        '''Check for required keys'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('required_keys', app=app)
            s = "OK"
            missing = []
            for f in self.required_keys:
                if not de.hasKey(f):
                    missing.append(f)
            if len(missing) > 0:
                t = 'error'
                s = 'missing required keys: %s' % ",".join(missing)
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('required_fields_not_empty', app=app)
            s = "OK"
            empty = []
            for f in self.required_keys:
                if de.hasKey(f) and de.get(f) == "":
                    empty.append(f)
            if len(empty) > 0:
                t = 'error'
                s = 'Empty required keys: %s' % ",".join(empty)
            self._add_result(t, n, s)

    def check_desktop_blacklisted_keys(self):
        '''Check for blacklisted keys'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('blacklisted_keys', app=app)
            s = "OK"
            found = []
            for f in self.blacklisted_keys:
                if de.hasKey(f):
                    found.append(f)
            if len(found) > 0:
                t = 'error'
                s = 'found blacklisted keys: %s' % ",".join(found)
            self._add_result(t, n, s)

    def check_desktop_exec(self):
        '''Check Exec entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Exec', app=app)
            s = 'OK'
            link = None
            if not de.hasKey('Exec'):
                t = 'error'
                s = "missing key 'Exec'"
            elif de.getExec().startswith('/'):
                t = 'error'
                s = "absolute path '%s' for Exec given in .desktop file." % \
                    de.getExec()
                link = ('http://askubuntu.com/questions/417381/'
                        'what-does-desktop-exec-mean/417382')
            elif de.getExec().split()[0] not in self.expected_execs:
                if self.pkg_arch[0] == "all":  # interpreted file
                    template = "found %s Exec with architecture '%s': %s"
                    if de.getExec().split()[0] not in self.deprecated_execs:
                        verb = 'unexpected'
                    else:
                        verb = 'deprecated'
                    s = template % \
                        (verb, self.pkg_arch[0], de.getExec().split()[0])
                    t = 'warn'
                else:                        # compiled
                    # TODO: this can be a lot smarter
                    s = "Non-standard Exec with architecture " + \
                        "'%s': %s (ok for compiled code)" % \
                        (self.pkg_arch[0], de.getExec().split()[0])
                    t = 'info'
            self._add_result(t, n, s, link)

    def check_desktop_exec_webapp_container(self):
        '''Check Exec=webapp-container entry'''
        if not self.is_click and not self.is_snap1:
            return

        if self.manifest is None:
            return

        fwk = self.manifest['framework']

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Exec_webapp_container', app=app)
            s = 'OK'
            if not de.hasKey('Exec'):
                t = 'error'
                s = "missing key 'Exec'"
                self._add_result(t, n, s)
                continue
            elif de.getExec().split()[0] == "ubuntu-html5-app-launcher" and \
                    fwk.startswith('ubuntu-sdk') and not \
                    (fwk.startswith('ubuntu-sdk-13') or
                     fwk.startswith('ubuntu-sdk-14')):
                # ubuntu-html5-app-launcher only available in ubuntu-sdk-14.10
                # and lower
                t = 'error'
                s = "ubuntu-html5-app-launcher is obsoleted in 15.04 " + \
                    "frameworks and higher. Please use 'webapp-container' " + \
                    "instead and ensure your security policy uses the " + \
                    "'ubuntu-webapp' template"
                self._add_result(t, n, s)
                continue
            elif de.getExec().split()[0] != "webapp-container":
                s = "SKIPPED (not webapp-container)"
                self._add_result(t, n, s)
                continue

            t = 'info'
            n = self._get_check_name('Exec_webapp_container_webapp', app=app)
            s = 'OK'
            if '--webapp' in de.getExec().split():
                t = 'error'
                s = "should not use --webapp in '%s'" % \
                    (de.getExec())
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('Exec_webapp_container_13.10', app=app)
            s = 'OK'
            if self.manifest['framework'] == "ubuntu-sdk-13.10":
                t = 'info'
                s = "'webapp-container' not available in 13.10 release " \
                    "images (ok if targeting 14.04 images with %s " \
                    "framework" % self.manifest['framework']
            self._add_result(t, n, s)

    def check_desktop_exec_webbrowser(self):
        '''Check Exec=webbrowser-app entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Exec_webbrowser', app=app)
            s = 'OK'
            if not de.hasKey('Exec'):
                t = 'error'
                s = "missing key 'Exec'"
                self._add_result(t, n, s)
                continue
            elif de.getExec().split()[0] != "webbrowser-app":
                s = "SKIPPED (not webbrowser-app)"
                self._add_result(t, n, s)
                continue

            t = 'info'
            n = self._get_check_name('Exec_webbrowser_webapp', app=app)
            s = 'OK'
            if '--webapp' not in de.getExec().split():
                t = 'error'
                s = "could not find --webapp in '%s'" % \
                    (de.getExec())
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('Exec_webbrowser_13.10', app=app)
            s = 'OK'
            if self.manifest['framework'] != "ubuntu-sdk-13.10":
                t = 'error'
                s = "may not use 'webbrowser-app' with framework '%s'" % \
                    self.manifest['framework']
            self._add_result(t, n, s)

    def check_desktop_exec_webapp_args(self):
        '''Check Exec=web* args'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Exec_webapp_args', app=app)
            s = 'OK'
            if not de.hasKey('Exec'):
                t = 'error'
                s = "missing key 'Exec'"
                self._add_result(t, n, s)
                continue
            elif de.getExec().split()[0] != "webbrowser-app" and \
                    de.getExec().split()[0] != "webapp-container":
                s = "SKIPPED (not webapp-container or webbrowser-app)"
                self._add_result(t, n, s)
                continue

            t = 'info'
            n = self._get_check_name('Exec_webapp_args_minimal_chrome',
                                     app=app)
            s = 'OK'
            if '--enable-back-forward' not in de.getExec().split():
                s = "could not find --enable-back-forward in '%s'" % \
                    (de.getExec())
            self._add_result(t, n, s)

            # verify the presence of either webappUrlPatterns or
            # webappModelSearchPath
            t = 'info'
            n = self._get_check_name('Exec_webapp_args_required', app=app)
            s = 'OK'
            found_url_patterns = False
            found_model_search_path = False
            found_named_webapp = False
            urls = []
            for i in de.getExec().split():
                if i == "webbrowser-app" or i == "webapp-container":
                    continue
                if i.startswith('--webappUrlPatterns'):
                    found_url_patterns = True
                if i.startswith('--webappModelSearchPath'):
                    found_model_search_path = True
                if i.startswith('--webapp='):
                    found_model_search_path = True
                # consider potential Exec field codes as 'non urls'
                if not i.startswith('--') and not i.startswith('%'):
                    urls.append(i)
            is_launching_local_app = True
            if len(urls) == 0:
                is_launching_local_app = False
            for url in urls:
                parts = urlsplit(url)
                if parts.scheme in ['http', 'https']:
                    is_launching_local_app = False
                    break
            if is_launching_local_app and \
                    (found_url_patterns or found_model_search_path or
                     found_named_webapp):
                t = 'error'
                s = "should not specify --webappUrlPatterns, " + \
                    "--webappModelSearchPath or --webapp= when " + \
                    "running local application"
            elif not is_launching_local_app:
                if not found_url_patterns and not found_model_search_path:
                    t = 'error'
                    s = "must specify one of --webappUrlPatterns or " + \
                        "--webappModelSearchPath"
            self._add_result(t, n, s)

    def _check_patterns(self, app, patterns, args):
        pattern_count = 1
        for pattern in patterns:
            urlp_scheme_pat = pattern[:-1].split(':')[0]
            urlp_p = urlsplit(re.sub(r'\?', '', pattern[:-1]))
            target = args[-1]
            urlp_t = urlsplit(target)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webapp_url_patterns_has_https', app=app,
                extra=pattern)
            s = 'OK'
            if not pattern.startswith('https?://'):
                t = 'warn'
                s = "'https?://' not found in '%s'" % pattern + \
                    " (may cause needless redirect)"
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webapp_url_patterns_uses_trailing_glob',
                app=app, extra=pattern)
            s = 'OK'
            if not pattern.endswith('*'):
                t = 'warn'
                s = "'%s' does not end with '*'" % pattern + \
                    " (may cause needless redirect) - %s" % urlp_p.path
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webapp_url_patterns_uses_unsafe_glob',
                app=app, extra=pattern)
            s = 'OK'
            if len(urlp_p.path) == 0 and pattern.endswith('*'):
                t = 'error'
                s = "'%s' contains trailing glob in netloc" % pattern
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webapp_url_patterns_uses_safe_glob',
                app=app, extra=pattern)
            s = 'OK'
            if '*' in pattern[:-1] and \
               (pattern[:-1].count('*') != 1 or
                    not pattern.startswith('https?://*')):
                t = 'warn'
                s = "'%s' contains nested '*'" % pattern + \
                    " (needs human review)"
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('Exec_webbrowser_target_exists', app=app)
            s = 'OK'
            if urlp_t.scheme == "":
                t = 'error'
                s = 'Exec line does not end with parseable URL'
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_target_scheme_matches_patterns',
                app=app, extra=pattern)
            s = 'OK'
            if not re.match(r'^%s$' % urlp_scheme_pat, urlp_t.scheme):
                t = 'error'
                s = "'%s' doesn't match '%s' " % (urlp_t.scheme,
                                                  urlp_scheme_pat) + \
                    "(will likely cause needless redirect)"
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_target_netloc_matches_patterns',
                app=app, extra=pattern)
            s = 'OK'
            # TODO: this is admittedly simple, but matches Canonical
            #       webapps currently, so ok for now
            if urlp_p.netloc.startswith('*') and len(urlp_p.netloc) > 2 and \
               urlp_t.netloc.endswith(urlp_p.netloc[1:]):
                s = "OK ('%s' matches '%s')" % (urlp_t.netloc, urlp_p.netloc)
            elif urlp_t.netloc != urlp_p.netloc:
                if pattern_count == 1:
                    t = 'warn'
                    s = "'%s' != primary pattern '%s'" % \
                        (urlp_t.netloc, urlp_p.netloc) + \
                        " (may cause needless redirect)"
                else:
                    t = 'info'
                    s = "target '%s' != non-primary pattern '%s'" % \
                        (urlp_t.netloc, urlp_p.netloc)
            self._add_result(t, n, s)

            pattern_count += 1

    def check_desktop_exec_webbrowser_urlpatterns(self):
        '''Check Exec=webbrowser-app entry has valid --webappUrlPatterns'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            execline = de.getExec().split()
            if not de.hasKey('Exec'):
                continue
            elif execline[0] != "webbrowser-app":
                continue
            elif len(execline) < 2:
                continue

            args = execline[1:]
            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webappUrlPatterns', app=app)
            s = 'OK'
            pats = ""
            count = 0
            for a in args:
                if not a.startswith('--webappUrlPatterns='):
                    continue
                pats = a.split('=', maxsplit=1)[1]
                count += 1

            if count == 0:
                # one of --webappUrlPatterns or --webappModelSearchPath is a
                # required arg and generates an error so just make this info
                t = 'info'
                s = "SKIPPED (--webappUrlPatterns not specified)"
                self._add_result(t, n, s)
                continue
            elif count > 1:
                t = 'error'
                s = "found multiple '--webappUrlPatterns=' in '%s'" % \
                    " ".join(args)
                self._add_result(t, n, s)
                continue

            self._check_patterns(app, pats.split(','), args)

    def _extract_webapp_manifests(self):
        '''Extract webapp manifest file'''
        files = sorted(glob.glob("%s/unity-webapps-*/manifest.json" %
                       self.unpack_dir))

        manifests = dict()
        for fn in files:
            key = os.path.relpath(fn, self.unpack_dir)
            try:
                manifests[key] = json.load(open_file_read(fn))
            except Exception:
                manifests[key] = None
                error("Could not parse '%s'" % fn, do_exit=False)

        return manifests

    def check_desktop_exec_webbrowser_modelsearchpath(self):
        '''Check Exec=webbrowser-app entry has valid --webappModelSearchPath'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            execline = de.getExec().split()
            if not de.hasKey('Exec'):
                continue
            elif execline[0] != "webbrowser-app":
                continue
            elif len(execline) < 2:
                continue

            args = execline[1:]
            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webappModelSearchPath_present', app=app)
            s = 'OK'
            path = ""
            count = 0
            for a in args:
                if not a.startswith('--webappModelSearchPath='):
                    continue
                path = a.split('=', maxsplit=1)[1]
                count += 1

            if count == 0:
                # one of --webappUrlPatterns or --webappModelSearchPath is a
                # required arg and generates an error so just make this info
                t = 'info'
                s = "SKIPPED (--webappModelSearchPath not specified)"
                self._add_result(t, n, s)
                continue
            elif count > 1:
                t = 'error'
                s = "found multiple '--webappModelSearchPath=' in '%s'" % \
                    " ".join(args)
                self._add_result(t, n, s)
                continue

            if not path:
                t = 'error'
                s = 'empty arg to --webappModelSearchPath'
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            # if --webappModelSearchPath is specified, that means we should
            # look for webapp configuration in the manifest.json in
            # ubuntu-webapps-*/
            manifests = self._extract_webapp_manifests()
            t = 'info'
            n = self._get_check_name(
                'Exec_webbrowser_webapp_manifest', app=app)
            s = 'OK'
            if len(manifests) == 0:
                t = 'error'
                s = 'could not find unity-webaps-*/manifest.json'
                self._add_result(t, n, s)
                continue
            elif len(manifests) > 1:
                # for now error on this since having
                # multiple manifests is unknown
                t = 'error'
                fns = []
                for f in manifests.keys():
                    fns.append(f)
                s = 'found multiple webapp manifest files: %s' % ",".join(fns)
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            for k in manifests.keys():
                m = manifests[k]

                t = 'info'
                n = self._get_check_name(
                    'Exec_webbrowser_webapp_manifest_wellformed', app=app,
                    extra=k)
                s = 'OK'
                if m is None or m == 'null':  # 'null' is for testsuite
                    t = 'error'
                    s = 'could not load webapp manifest file. Is it ' + \
                        'properly formatted?'
                    self._add_result(t, n, s)
                    continue
                self._add_result(t, n, s)

                # 'includes' contains the patterns
                t = 'info'
                n = self._get_check_name(
                    'Exec_webbrowser_webapp_manifest_includes_present',
                    app=app, extra=k)
                s = 'OK'
                if 'includes' not in m:
                    t = 'error'
                    s = "could not find 'includes' in webapp manifest"
                elif not isinstance(m['includes'], list):
                    t = 'error'
                    s = "'includes' in webapp manifest is not list"
                self._add_result(t, n, s)
                if t == 'error':
                    continue

                self._check_patterns(app, m['includes'], args)

    def check_desktop_groups(self):
        '''Check Desktop Entry entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('groups', app=app)
            s = "OK"
            if len(de.groups()) != 1:
                t = 'error'
                s = 'too many desktop groups'
            elif "Desktop Entry" not in de.groups():
                t = 'error'
                s = "'[Desktop Entry]' group not found"
            self._add_result(t, n, s)

    def check_desktop_type(self):
        '''Check Type entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Type', app=app)
            s = "OK"
            if not de.hasKey('Type'):
                t = 'error'
                s = "missing key 'Type'"
            elif de.getType() != "Application":
                t = 'error'
                s = 'does not use Type=Application'
            self._add_result(t, n, s)

    def check_desktop_x_ubuntu_touch(self):
        '''Check X-Ubuntu-Touch entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('X-Ubuntu-Touch', app=app)
            s = "OK"
            if not de.hasKey('X-Ubuntu-Touch'):
                t = 'error'
                s = "missing key 'X-Ubuntu-Touch'"
            elif de.get("X-Ubuntu-Touch") != "true" and \
                    de.get("X-Ubuntu-Touch") != "True":
                t = 'error'
                s = 'does not use X-Ubuntu-Touch=true'
            self._add_result(t, n, s)

    def check_desktop_x_ubuntu_stagehint(self):
        '''Check X-Ubuntu-StageHint entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('X-Ubuntu-StageHint', app=app)
            s = "OK"
            if not de.hasKey('X-Ubuntu-StageHint'):
                t = 'info'
                s = "OK (not specified)"
            elif de.get("X-Ubuntu-StageHint") != "SideStage":
                t = 'error'
                s = "unsupported X-Ubuntu-StageHint=%s " % \
                    de.get("X-Ubuntu-StageHint") + \
                    "(should be for example, 'SideStage')"
            self._add_result(t, n, s)

    def check_desktop_x_ubuntu_gettext_domain(self):
        '''Check X-Ubuntu-Gettext-Domain entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('X-Ubuntu-Gettext-Domain', app=app)
            s = "OK"
            if not de.hasKey('X-Ubuntu-Gettext-Domain'):
                t = 'info'
                s = "OK (not specified)"
            elif de.get("X-Ubuntu-Gettext-Domain") == "":
                t = 'error'
                s = "X-Ubuntu-Gettext-Domain is empty"
            elif de.get("X-Ubuntu-Gettext-Domain") != self.click_pkgname:
                t = 'warn'
                s = "'%s' != '%s'" % (de.get("X-Ubuntu-Gettext-Domain"),
                                      self.click_pkgname)
                s += " (ok if app uses i18n.domain('%s')" % \
                     de.get("X-Ubuntu-Gettext-Domain") + \
                     " or uses organizationName"
            self._add_result(t, n, s)

    def check_desktop_terminal(self):
        '''Check Terminal entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Terminal', app=app)
            s = "OK"
            if not de.hasKey('Terminal'):
                s = "OK (not specified)"
            elif de.getTerminal() is not False:
                t = 'error'
                s = 'does not use Terminal=false (%s)' % de.getTerminal()
            self._add_result(t, n, s)

    def check_desktop_version(self):
        '''Check Version entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Version', app=app)
            s = "OK"
            link = None
            if not de.hasKey('Version'):
                s = "OK (not specified)"
            elif de.getVersionString() not in self.supported_versions:
                # http://standards.freedesktop.org/desktop-entry-spec/latest
                t = 'error'
                s = "'%s' does not match any freedesktop.org version %s" % \
                    (de.getVersionString(), self.supported_versions)
                link = ('http://askubuntu.com/questions/419907/'
                        'what-does-version-mean-in-the-desktop-file/419908')
            self._add_result(t, n, s, link)

    def check_desktop_comment(self):
        '''Check Comment entry'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Comment_boilerplate', app=app)
            s = "OK"
            link = None
            if de.hasKey('Comment') and \
                    de.getComment() == "My project description":
                t = 'warn'
                s = "Comment uses SDK boilerplate '%s'" % de.getComment()
                link = ('http://askubuntu.com/questions/417359/'
                        'what-does-desktop-comment-boilerplate-mean/417360')
            self._add_result(t, n, s, link)

    def check_desktop_icon(self):
        '''Check Icon entry'''
        if not self.is_click and not self.is_snap1:
            return

        ICON_SUFFIXES = ['.svg',
                         '.png',
                         '.jpg',
                         ]
        for app in sorted(self.desktop_entries):
            de = self._get_desktop_entry(app)
            t = 'info'
            n = self._get_check_name('Icon', app=app)
            s = 'OK'
            link = None
            if not de.hasKey('Icon'):
                t = 'error'
                s = "missing key 'Icon'"
                link = ('http://askubuntu.com/questions/417369/'
                        'what-does-desktop-icon-mean/417370')
            elif de.getIcon().startswith('/'):
                t = 'error'
                s = "absolute path '%s' for icon given in .desktop file." % \
                    de.getIcon()
                link = ('http://askubuntu.com/questions/417369/'
                        'what-does-desktop-icon-mean/417370')
            elif not os.path.exists(os.path.join(self.unpack_dir,
                                                 de.getIcon())) and \
                    not any(map(lambda a:
                                os.path.exists(os.path.join(
                                               self.unpack_dir,
                                               de.getIcon() + a)),
                                ICON_SUFFIXES)):
                t = 'error'
                s = "'%s' specified as icon in .desktop file for app '%s', " \
                    "which is not available in the click package." % \
                    (de.getIcon(), app)
                link = ('http://askubuntu.com/questions/417369/'
                        'what-does-desktop-icon-mean/417370')
            self._add_result(t, n, s, link)

    def check_desktop_duplicate_entries(self):
        '''Check desktop for duplicate entries'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.desktop_entries):
            found = []
            dupes = []
            t = 'info'
            n = self._get_check_name('duplicate_keys', app=app)
            s = 'OK'
            fn = self._get_desktop_filename(app)
            content = open_file_read(fn).readlines()
            for line in content:
                tmp = line.split('=')
                if len(tmp) < 2:
                    continue
                if tmp[0] in found:
                    dupes.append(tmp[0])
                else:
                    found.append(tmp[0])
            if len(dupes) > 0:
                t = 'error'
                s = 'found duplicate keys: %s' % ",".join(dupes)
            self._add_result(t, n, s)
